@@ -1,16 +1,17 @@
 package Lingua::JA::WordNet;
 
-use 5.008_001;
+use 5.008_008;
 use strict;
 use warnings;
 
 use DBI;
 use Carp ();
-use File::ShareDir ();
+use File::ShareDir  ();
+use List::MoreUtils ();
 
-our $VERSION = '0.12';
+our $VERSION = '0.20';
 
-my $DB_FILE = 'wnjpn-1.1.db';
+my $DB_FILE = 'wnjpn-1.1_and_synonyms-1.0.db';
 
 
 sub _options
@@ -75,7 +76,7 @@ sub Word
 
     $sth->execute($synset, $lang);
 
-    my @words = map { $_->[0] =~ s/_/ /g; $_->[0]; } @{$sth->fetchall_arrayref};
+    my @words = map { $_->[0] =~ tr/_/ /; $_->[0]; } @{$sth->fetchall_arrayref};
 
     Carp::carp "Word: there are no words for $synset in $lang" if $self->{verbose} && ! scalar @words;
 
@@ -100,7 +101,7 @@ sub Synset
 
     my @synsets = map {$_->[0]} @{$sth->fetchall_arrayref};
 
-    Carp::carp "Synset: there are no synsets for $word in $lang" if $self->{verbose} && ! scalar @synsets;
+    Carp::carp "Synset: there are no synsets for '$word' in $lang" if $self->{verbose} && ! scalar @synsets;
 
     return @synsets;
 }
@@ -124,7 +125,7 @@ sub SynPos
 
     my @synsets = map {$_->[0]} @{$sth->fetchall_arrayref};
 
-    Carp::carp "SynPos: there are no synsets for $word corresponding to '$pos' and '$lang'" if $self->{verbose} && ! scalar @synsets;
+    Carp::carp "SynPos: there are no synsets for '$word' corresponding to '$pos' and '$lang'" if $self->{verbose} && ! scalar @synsets;
 
     return @synsets;
 }
@@ -225,6 +226,53 @@ sub AllSynsets
     return \@synsets;
 }
 
+sub WordID
+{
+    my ($self, $word, $pos, $lang) = @_;
+
+    $word =~ tr/ /_/;
+    $lang = 'jpn' unless defined $lang;
+
+    my $sth
+        = $self->{dbh}->prepare
+        (
+            'SELECT wordid FROM word
+              WHERE lemma = ?
+                AND pos   = ?
+                AND lang  = ?'
+        );
+
+    $sth->execute($word, $pos, $lang);
+
+    my ($wordid) = map {$_->[0]} @{$sth->fetchall_arrayref};
+
+    Carp::carp "WordID: there is no WordID for '$word' corresponding to '$pos' and '$lang'" if $self->{verbose} && ! defined $wordid;
+
+    return $wordid;
+}
+
+sub Synonym
+{
+    my ($self, $wordid) = @_;
+
+    my $sth
+        = $self->{dbh}->prepare
+        (
+            'SELECT lemma FROM word JOIN wordlink ON word.wordid = wordlink.wordid2
+              WHERE wordid1 = ?
+                AND link    = ?'
+        );
+
+    $sth->execute($wordid, 'syns');
+    my @synonyms = map {$_->[0]} @{$sth->fetchall_arrayref};
+
+    Carp::carp "Synonyms: there are no Synonyms for $wordid" if $self->{verbose} && ! scalar @synonyms;
+
+    # 一応順番を保持したいのでハッシュスライスは使わない
+    # uniq: The order of elements in the returned list is the same as in LIST.
+    return List::MoreUtils::uniq @synonyms;
+}
+
 1;
 
 __END__
@@ -250,6 +298,13 @@ my ($db_path, %config, $synset, $lang, $pos, $rel);
   print "$words[0]\n";
   # -> レスリング
 
+  # Synonym method can access to Japanese WordNet Synonyms Database.
+  my $wordID   = $wn->WordID('ねんねこ', 'n');
+  my @synonyms = $wn->Synonym($wordID);
+
+  print "@synonyms\n";
+  # -> お休み ねね スリープ 就眠 御休み 眠り 睡り 睡眠
+
 =head1 DESCRIPTION
 
 Japanese WordNet is a semantic dictionary of Japanese.
@@ -267,7 +322,7 @@ Because of this, I uploaded this module.
 Creates a new Lingua::JA::WordNet instance.
 
   my $wn = Lingua::JA::WordNet->new(
-      data        => $db_path, # default is File::ShareDir::dist_file('Lingua-JA-WordNet', 'wnjpn-1.1.db')
+      data        => $db_path, # default is File::ShareDir::dist_file('Lingua-JA-WordNet', 'wnjpn-1.1_and_synonyms-1.0.db')
       enable_utf8 => 1,        # default is 0 (see sqlite_unicode attribute of DBD::SQLite)
       verbose     => 0,        # default is 0 (all warnings are ignored)
   );
@@ -307,6 +362,16 @@ Returns the example sentences corresponding to $synset and $lang,
 
 Returns all synsets.
 
+=head2 $wordID = $wn->WordID( $word, $pos [, $lang] )
+
+Returns the word ID corresponding to $word, $pos and $lang.
+
+=head2 @synonyms = $wn->Synonym($wordID)
+
+Returns the synonyms of $wordID.
+
+This method works only under the bundled Japanese WordNet database file.
+
 
 =head2 LANGUAGES
 
@@ -362,6 +427,12 @@ $rel can take the left side values of the following table.
 This is the result of the SQL query 'SELECT link, def FROM link_def'.
 
 
+=head1 Out of memory!
+
+In rare cases, this error message is displayed during the installation of this library.
+If this is displayed, please install this library manually. (RT#82276)
+
+
 =head1 AUTHOR
 
 pawa E<lt>pawapawa@cpan.orgE<gt>
@@ -374,7 +445,7 @@ L<http://twitter.com/LinguaJAWordNet>
 
 =head1 LICENSE
 
-This library except bundled WordNet database file is free software; you can redistribute it and/or modify
+This library except the bundled WordNet database file is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 The bundled WordNet database file complies with the following licenses:
